
#
# COMP 304 2009
#
# 2007 Alexandre Denault
# 2008, 2009 small updates by Hans Vangheluwe
#

from   Tkinter      import *  # basic Tkinter
import tkFileDialog           # file selection (to output Postscript in)
import tkMessageBox
import tkFont

#"Sans Serif font"
#SS_FONT=("helvetica", 10)

import GridWindow

#####
# This class is used to create the controls for the Sudoku GUI. Note
# that this is the root class of this GUI. This means that if this
# window is closed, all the other windows are also closed.
#
class SudokuGUI:
    
    # Initializes the GUI interface
    def __init__(self, master):
    
        self.master = master
        self.master.protocol("WM_DELETE_WINDOW", self.onQuit)
    
        frame = Frame(master)

        fileFrame = Frame(frame, borderwidth=5)
        self.btn_load = Button(fileFrame, width=15, text="Load from File", command=self.load)
        self.btn_load.config(state="active")
        self.btn_load.pack()
        self.btn_saveAs = Button(fileFrame, width=15, text="Save As", command=self.saveAs)
        self.btn_saveAs.config(state="active")
        self.btn_saveAs.pack()
        self.btn_save = Button(fileFrame, width=15, text="Save", command=self.save)
        self.btn_save.config(state="disabled") # disabled as long as no filename set (by previous "Save As")
        self.btn_save.pack()
        fileFrame.pack()

        undoFrame = Frame(frame, borderwidth=5)
        self.btn_undo = Button(undoFrame, width=15, text="Undo last", command=self.undo)
        self.btn_undo.pack()
        self.btn_undo_since_error = Button(undoFrame, width=15, text="Undo since error", command=self.undo_last_error)
        self.btn_undo_since_error.pack()
        undoFrame.pack()

        topFrame = Frame(frame, borderwidth=5)
        self.btn_open = Button(topFrame, width=15, text="Open Full Board", command=self.open_main_board)
        self.btn_open.pack()
        topFrame.pack()
        
        blockFrame = Frame(frame, borderwidth=5)
        
        frame_label = Frame(blockFrame)
        label = Label(frame_label, text="Open Block Windows")
        label.pack()
        frame_label.pack()
        
        frame_buttons = Frame(blockFrame)
        self.btn_open_block1 = Button(frame_buttons, text="(1,1)", command= lambda : self.open_block_board(0,0) )
        self.btn_open_block1.grid(row=0, column=0)
        self.btn_open_block2 = Button(frame_buttons, text="(1,2)", command= lambda : self.open_block_board(0,3) )
        self.btn_open_block2.grid(row=0, column=1)
        self.btn_open_block3 = Button(frame_buttons, text="(1,3)", command= lambda : self.open_block_board(0,6) )
        self.btn_open_block3.grid(row=0, column=2)
        self.btn_open_block4 = Button(frame_buttons, text="(2,1)", command= lambda : self.open_block_board(3,0) )
        self.btn_open_block4.grid(row=1, column=0)
        self.btn_open_block5 = Button(frame_buttons, text="(2,2)", command= lambda : self.open_block_board(3,3) )
        self.btn_open_block5.grid(row=1, column=1)
        self.btn_open_block6 = Button(frame_buttons, text="(2,3)", command= lambda : self.open_block_board(3,6) )
        self.btn_open_block6.grid(row=1, column=2)
        self.btn_open_block7 = Button(frame_buttons, text="(3,1)", command= lambda : self.open_block_board(6,0) )
        self.btn_open_block7.grid(row=2, column=0)
        self.btn_open_block8 = Button(frame_buttons, text="(3,2)", command= lambda : self.open_block_board(6,3) )
        self.btn_open_block8.grid(row=2, column=1)
        self.btn_open_block9 = Button(frame_buttons, text="(3,3)", command= lambda : self.open_block_board(6,6) )
        self.btn_open_block9.grid(row=2, column=2)
        frame_buttons.pack()
        
        blockFrame.pack()
        
        rowFrame = Frame(frame, borderwidth=5)
        
        frame_label = Frame(rowFrame)
        label = Label(frame_label, text="Open Row Windows")
        label.pack()
        frame_label.pack()
        
        frame_buttons = Frame(rowFrame)
        self.btn_open_block1 = Button(frame_buttons, text=" 1 ", command= lambda : self.open_row_board(0,0) )
        self.btn_open_block1.grid(row=0, column=0)
        self.btn_open_block2 = Button(frame_buttons, text=" 2 ", command= lambda : self.open_row_board(1,0) )
        self.btn_open_block2.grid(row=0, column=1)
        self.btn_open_block3 = Button(frame_buttons, text=" 3 ", command= lambda : self.open_row_board(2,0) )
        self.btn_open_block3.grid(row=0, column=2)
        self.btn_open_block4 = Button(frame_buttons, text=" 4 ", command= lambda : self.open_row_board(3,0) )
        self.btn_open_block4.grid(row=1, column=0)
        self.btn_open_block5 = Button(frame_buttons, text=" 5 ", command= lambda : self.open_row_board(4,0) )
        self.btn_open_block5.grid(row=1, column=1)
        self.btn_open_block6 = Button(frame_buttons, text=" 6 ", command= lambda : self.open_row_board(5,0) )
        self.btn_open_block6.grid(row=1, column=2)
        self.btn_open_block7 = Button(frame_buttons, text=" 7 ", command= lambda : self.open_row_board(6,0) )
        self.btn_open_block7.grid(row=2, column=0)
        self.btn_open_block8 = Button(frame_buttons, text=" 8 ", command= lambda : self.open_row_board(7,0) )
        self.btn_open_block8.grid(row=2, column=1)
        self.btn_open_block9 = Button(frame_buttons, text=" 9 ", command= lambda : self.open_row_board(8,0) )
        self.btn_open_block9.grid(row=2, column=2)
        frame_buttons.pack()
        
        rowFrame.pack()
        
        colFrame = Frame(frame, borderwidth=5)
        
        frame_label = Frame(colFrame)
        label = Label(frame_label, text="Open Column Windows")
        label.pack()
        frame_label.pack()
        
        frame_buttons = Frame(colFrame)
        self.btn_open_block1 = Button(frame_buttons, text=" 1 ", command= lambda : self.open_col_board(0,0) )
        self.btn_open_block1.grid(row=0, column=0)
        self.btn_open_block2 = Button(frame_buttons, text=" 2 ", command= lambda : self.open_col_board(0,1) )
        self.btn_open_block2.grid(row=0, column=1)
        self.btn_open_block3 = Button(frame_buttons, text=" 3 ", command= lambda : self.open_col_board(0,2) )
        self.btn_open_block3.grid(row=0, column=2)
        self.btn_open_block4 = Button(frame_buttons, text=" 4 ", command= lambda : self.open_col_board(0,3) )
        self.btn_open_block4.grid(row=1, column=0)
        self.btn_open_block5 = Button(frame_buttons, text=" 5 ", command= lambda : self.open_col_board(0,4) )
        self.btn_open_block5.grid(row=1, column=1)
        self.btn_open_block6 = Button(frame_buttons, text=" 6 ", command= lambda : self.open_col_board(0,5) )
        self.btn_open_block6.grid(row=1, column=2)
        self.btn_open_block7 = Button(frame_buttons, text=" 7 ", command= lambda : self.open_col_board(0,6) )
        self.btn_open_block7.grid(row=2, column=0)
        self.btn_open_block8 = Button(frame_buttons, text=" 8 ", command= lambda : self.open_col_board(0,7) )
        self.btn_open_block8.grid(row=2, column=1)
        self.btn_open_block9 = Button(frame_buttons, text=" 9 ", command= lambda : self.open_col_board(0,8) )
        self.btn_open_block9.grid(row=2, column=2)
        frame_buttons.pack()
        
        colFrame.pack()
        
        bottomFrame = Frame(frame, borderwidth=5)
        self.btn_quit = Button(bottomFrame, text="Quit", command=self.quit)
        self.btn_quit.pack()
        bottomFrame.pack()
        
        frame.pack()
    
    # This method is called when the "Open Full Board" button is pressed.
    def open_main_board(self):
        newWindow = GridWindow.MainWindow(self.master)

    # This method is called when one of the "Open Block Board" buttons is pressed. 
    def open_block_board(self, rowOffset, colOffset):
        newWindow = GridWindow.BlockWindow(self.master, rowOffset, colOffset)

    # This method is called when one of the "Open Row Board" buttons is pressed. 
    def open_row_board(self, rowOffset, colOffset):
        newWindow = GridWindow.RowWindow(self.master, rowOffset, colOffset)

    # This method is called when one of the "Open Column Board" buttons is pressed. 
    def open_col_board(self, rowOffset, colOffset):
        newWindow = GridWindow.ColWindow(self.master, rowOffset, colOffset)
                    
    # This method is called when the "Load from File" button is pressed.
    # It opens a file dialog to choose the file to load from 
    # and loads a Sudoku configuration from it
    def load(self):
        print "Loading from File"        
        fileName = tkFileDialog.askopenfilename(defaultextension=".sdk",
                                                filetypes=[("Sudoku", "*.sdk"),
                                                           ("All", "*")])
        #if not self.subject.importFromFile(fileName):
        if False:
          tkMessageBox.showwarning("Load", "Unable to load the file")
        else:
          self.__lastFileName = fileName
          self.btn_save.config(state="active")     

    # This method is called when the "Save As" button is pressed.
    # It opens a file dialog to choose the file to save to 
    # and saves the current Sudoku configuration to it.
    def saveAs(self):
        print "Saving to File"        
        fileName = tkFileDialog.asksaveasfilename(defaultextension=".sdk",
                                                  filetypes=[("Sudoku", "*.sdk"),
                                                             ("All", "*")])
        #if not self.subject.exportToFile(fileName):
        if False:
          tkMessageBox.showwarning("Save As", "Unable to create the file")
        else:
          self.__lastFileName = fileName
          self.btn_save.config(state="active")

    # This method is called when the "Save" button is pressed.
    # The button is disactivated if no filename is known.
    # A filename is known when either "Save As" or "Load" (from) were previously done.
    # In that case, the Sudoku configuration is saved to this
    # previously specified file.
    def save(self):
        print "Saving to File"        
        #if not self.subject.exportToFile(elf.__savedFileName):
        if True:
          tkMessageBox.showwarning("Save As", "Unable to create the file")

    # This method is called when the "Undo" button is pressed.
    def undo(self):
        print "Undoing"        
    
    # This method is called when the "Undo Last Error" button is pressed.
    def undo_last_error(self):
        print "Undoing since last error"
    
    # This method is called when the "Quit" button is pressed
    def quit(self):
        self.master.destroy()
        
    # This method is called when the window is closed.
    def onQuit(self):
        self.master.destroy()


class SudokuBoardState:

  def __init__(self):

    self.state = {}
    for row in range(9):
      for col in range(9):
        self.state[(row,col)] = 0 # 0 denotes empty 

  def __str__(self):
    strRepr = ""
    for row in range(9):
      for col in range(9):
        strRepr += " "+str(self.state[(row,col)])
      strRepr += "\n"
    return strRepr

  # add get and set methods for individual cells


boardState = SudokuBoardState()
print "boardState\n"
print boardState

root = Tk()
sudokuGui = SudokuGUI(root)

root.title('Ultimate Sudoku')
root.mainloop()

